import codecs
import logging

import os
import xml.etree.ElementTree as etree

from babel.messages.pofile import read_po

log = logging.getLogger(__name__)


def translate_desktop_file(infile, outfile, localedir):
    infp = codecs.open(infile, 'rb', encoding='utf-8')
    outfp = codecs.open(outfile, 'wb', encoding='utf-8')

    catalogs = get_catalogs(localedir)

    for line in (x.strip() for x in infp):
        log.debug('Found in original (%s): %r', type(line), line)
        # We intend to ignore the first line
        if line.startswith('[Desktop'):
            additional_lines = []
        else:
            additional_lines = []
            # This is a rather primitive approach to generating the translated
            # desktop file.  For example we don't really care about all the
            # keys in the file.  But its simplicity is a feature and we
            # ignore the runtime overhead, because it should only run centrally
            # once.
            key, value = line.split('=', 1)
            log.debug("Found key: %r", key)
            for locale, catalog in catalogs.items():
                translated = catalog.get(value)
                log.debug("Translated %r[%r]=%r: %r (%r)",
                          key, locale, value, translated,
                          translated.string if translated else '')
                if translated and translated.string \
                              and translated.string != value:
                    additional_line = u'{keyword}[{locale}]={translated}'.format(
                                        keyword=key,
                                        locale=locale,
                                        translated=translated.string,
                                    )
                    additional_lines.append(additional_line)
                log.debug("Writing more lines: %s", additional_lines)

        # Write the new file.
        # First the original line found it in the file, then the translations.
        outfp.writelines((outline+'\n' for outline in ([line] + additional_lines)))

    infp.close()
    outfp.close()


def translate_appdata_file(infile, outfile, localedir):
    catalogs = get_catalogs(localedir)
    tree = etree.parse(infile)
    root = tree.getroot()
    add_translations(root, catalogs)
    tree.write(outfile, encoding='utf-8', xml_declaration=True)


def add_translations(parent, catalogs):
    tail = parent.text
    last_tail = None
    for pos, elem in enumerate(parent, start=1):
        if elem.get("translatable") == "yes":
            elem.attrib.pop("translatable", None)
            elem.attrib.pop("comments", None)  # Are comments allowed?
            last_tail = elem.tail
            elem.tail = tail
            message = elem.text
            for locale, catalog in catalogs.items():
                translated = catalog.get(message)
                if translated and translated.string \
                        and translated.string != message:
                    log.debug("Translated [%s]%r: %r (%r)",
                              locale, message, translated, translated.string)
                    tr = etree.Element(elem.tag)
                    attrib = tr.attrib
                    attrib["{http://www.w3.org/XML/1998/namespace}lang"] = str(locale)
                    tr.text = translated.string
                    tr.tail = tail
                    parent.insert(pos, tr)
        else:
            add_translations(elem, catalogs)
    if last_tail:
        parent[-1].tail = last_tail

def get_catalogs(localedir):
    # glob in Python 3.5 takes ** syntax
    # pofiles = glob.glob(os.path.join(localedir, '**.po', recursive=True))
    pofiles = sorted([os.path.join(dirpath, f)
               for dirpath, dirnames, files in os.walk(localedir)
               for f in files if f.endswith('.po')])
    logging.debug('Loading %r', pofiles)
    catalogs = {}

    for pofile in pofiles:
        with open(pofile, 'r') as f:
            catalog = read_po(f)
        catalogs[catalog.locale] = catalog
        logging.info("Found %d strings for %s", len(catalog), catalog.locale)
        # logging.debug("Strings for %r", catalog, catalog.values())
    if not catalogs:
        logging.warning("Could not find pofiles in %r", pofiles)
    return catalogs



# -*- coding: utf-8 -*-
# vim: sw=4 ts=4 fenc=utf-8
# =============================================================================
# $Id$
# =============================================================================
#             $URL$
# $LastChangedDate$
#             $Rev$
#   $LastChangedBy$
# =============================================================================
# Copyright (C) 2006 Ufsoft.org - Pedro Algarvio <ufs@ufsoft.org>
#
# Please view LICENSE for additional licensing information.
# =============================================================================
from __future__ import unicode_literals

import xml.etree.ElementTree as etree


def extract_glade(fileobj, keywords, comment_tags, options):
    """Extracts translatable strings from Glade files or GtkBuilder UI XML.

    :param fileobj: the file-like object to extract from, iterable by lines
    :param keywords: a list of translation keywords to extract, with the same
        names and meanings as C/Python i18n function names.
    :param comment_tags: a list of translator tags to search for and
        include in the results. This is ignored.
    :param options: a dictionary of additional options (optional)
    :return: An iterator over ``(lineno, funcname, message, comments)``
        tuples whose interpretation depends on ``funcname``.
    :rtype: iterator

    Properties must be marked translatable="yes". Context and comments
    attributes are respected. The yielded tuples are and returned as if
    you used gettext() or pgettext() in C or Python code. In other
    words, "gettext" or "pgettext" must be listed in ``keywords`` for
    contextless or context-bearing strings to be translated. The
    shorthand "_" and "C_" aliases from g18n.h are valid keywords too.

    Babel defaults to having "gettext" and "pgettext" in ``keywords``,
    so you don't normally need to worry about this.

    See also:

    * babel.messages.extract.extract()
    * http://babel.pocoo.org/en/latest/messages.html#writing-extraction-methods
    * https://www.gnu.org/software/gettext/manual/html_node/PO-Files.html

    """
    parser = etree.XMLPullParser(["end"])
    pgettext_wanted = ("pgettext" in keywords) or ("C_" in keywords)
    gettext_wanted = "gettext" in keywords
    truthy_values = [s.casefold() for s in ["yes", "true", "1", "y", "t"]]
    for line_idx, line_data in enumerate(fileobj):
        parser.feed(line_data)
        for event, elem in parser.read_events():
            assert event == "end"
            translatable_attr = elem.attrib.get("translatable", "no")
            if not translatable_attr.casefold() in truthy_values:
                continue

            comments = []
            if "comments" in elem.attrib:
                comments.append(elem.attrib["comments"])

            # Babel's interpretation of the yielded tuple depends on the
            # function name returned as part of it. This tells Babel what
            # the elements of the returned messages list or tuple mean.
            func_name = None
            if "context" in elem.attrib and pgettext_wanted:
                func_name = "pgettext"
                context = elem.attrib["context"]
                messages = [context, elem.text]
            elif gettext_wanted and "context" not in elem.attrib:
                # Returned strings are equivalent to a list or tuple
                # of length 1, like the arguments to C gettext()/_().
                func_name = "gettext"
                messages = elem.text

            if func_name is None:
                continue
            yield (line_idx + 1, func_name, messages, comments)


# All localestrings from https://specifications.freedesktop.org/desktop-entry-spec/latest/ar01s05.html
TRANSLATABLE = (
    'Name',
    'GenericName',
    'Comment',
    'Icon',
    'Keywords',
)


def extract_desktop(fileobj, keywords, comment_tags, options):
    for lineno, line in enumerate(fileobj, 1):
        if line.startswith(b'[Desktop Entry]'):
            continue

        for t in TRANSLATABLE:
            if not line.startswith(t.encode('utf-8')):
                continue
            else:
                l = line.decode('utf-8')
                comments = []
                key_value = l.split('=', 1)
                key, value = key_value[0:2]

                funcname = key # FIXME: Why can I not assign that name to funcname?
                funcname = ''
                message = value
                comments.append(key)
                yield (lineno, funcname, message.strip(), comments)
